@@ -8,7 +8,7 @@ sp_tables_i <- tibble::tribble(~table_num, ~report_num, ~id,   ~table_code,   ~d
                                0,           0,          "cash-flow",  "ppt", "ppt", "PPT", "P\\u0159ehled pen\\u011b\\u017en\\u00edch tok\\u016f", NA,
                                0,           0,          "balance-sheet",     "rozvaha1",    "rozv",      "ROZV[1]?",      "Rozvaha - prvn\\u00ed \\u010d\\u00e1st", NA,
                                0,           0,          "balance-sheet-2",     "rozvaha2",    "rozv",      "ROZV2",      "Rozvaha - druh\\u00e1 \\u010d\\u00e1st", NA,
-                               0,           0,          "balance-sheet-city-districts",     "rozvaha1mc",  "rozv",      "ROZV1MC",    "Rozvaha m\\u011bstsk\\u00fdch \\u010d\\u00e1st\\u00ed - prvn\\u00ed \\u010d\\u00e1st", "only for 2018; in other years city districts are incorporated in balance sheet",
+                               0,           0,          "balance-sheet-city-districts",     "rozvaha1mc",  "rozv",      "ROZV[1_]MC",    "Rozvaha m\\u011bstsk\\u00fdch \\u010d\\u00e1st\\u00ed - prvn\\u00ed \\u010d\\u00e1st", "only for 2018; in other years city districts are incorporated in balance sheet",
                                0,           0,          "balance-sheet-city-districts-2",     "rozvaha2mc",  "rozv",      "ROZVMC2",    "Rozvaha m\\u011bstsk\\u00fdch \\u010d\\u00e1st\\u00ed - druh\\u00e1 \\u010d\\u00e1st", "only for 2018; in other years city districts are incorporated in balance sheet",
                                0,           0,          "budget-central-old",     "finu_budget",  "finu",      "FINU101",    "Pln\\u011bn\\u00ed rozpo\\u010dtu \\u00fast\\u0159edn\\u011b \\u0159\\u00edzen\\u00fdch organizac\\u00ed do 2014", "pre-2015 only",
                                0,           0,          "budget-central-old-purpose-grants",     "finu_ucel",  "finu",      "FINU107",   "\\u00da\\u010delov\\u00e9 financov\\u00e1n\\u00ed - poskytnut\\u00e9 prost\\u0159edky", "pre-2015 only",
@@ -38,6 +38,130 @@ sp_tables_i <- tibble::tribble(~table_num, ~report_num, ~id,   ~table_code,   ~d
   dplyr::mutate_if(is.character, stringi::stri_unescape_unicode)
 # usethis::use_data(sp_tables, overwrite = TRUE)
 
+
+#' Get path to a CSV file containing a table.
+#'
+#' This is normally called inside `sp_get_table()` but can be used separately if
+#' finer-grained control of intermediate outputs is needed, e.g. in a {targets} workflow.
+#'
+#' @param table_id Table ID; see `id` column in `sp_tables` for a list of available codelists.
+#' @param dataset_path Path to downloaded dataset, as output by `sp_get_dataset()`
+#' @param reunzip Whether to overwrite existing CSV files by unzipping the archive downlaoded by `sp_get_dataset()`. Defaults to FALSE.
+#'
+#' @return Character vector of length one - a path.
+#' @family Detailed workflow
+#' @examples
+#' \dontrun{
+#' ds <- sp_get_dataset("rozv", 2018, 12)
+#' sp_get_table_file("balance-sheet", ds)
+#' }
+#' @export
+sp_get_table_file <- function(table_id, dataset_path, reunzip = FALSE) {
+  stopifnot(file.exists(dataset_path))
+  dd <- dirname(dataset_path)
+  dslist <- list.files(dd, pattern = "*.CSV|*.csv", full.names = T)
+  if(length(dslist) == 0 | reunzip) {
+    usethis::ui_info("Unzipping...")
+    dslist <- utils::unzip(dataset_path, exdir = dd)
+  }
+
+  table_regex <- paste0(sp_tables_i$file_regex[sp_tables_i$id == table_id])
+
+  table_file <- dslist[stringr::str_detect(dslist,
+                                           paste0(table_regex, "(_[0-9]*)?\\.(csv|CSV)"))]
+  if(length(table_file) != 1) {
+    if (length(table_file) > 1) {
+      ui_stop(c("More than one CSV files in the archive match.",
+                         "You might want to report a bug at {ui_path('https://github.com/petrbouchal/statnipokladna/issues')}."))
+    }  else {
+      ui_stop(c("No CSV file inside the downloaded archive matches files needed for the table.",
+                         "You might want to report a bug at {ui_path('https://github.com/petrbouchal/statnipokladna/issues')}."))
+    }
+  }
+  return(table_file)
+}
+
+
+#' Load a statnipokladna table from a CSV file
+#'
+#' This is normally called inside `sp_get_table()` but can be used separately if
+#' finer-grained control of intermediate outputs is needed, e.g. in a {targets} workflow.
+#'
+#' @param path path to a CSV file, as output by `sp_get_table_file()`.
+#' @param ico Organisation ID to filter by, if supplied.
+#'
+#' @return a [tibble][tibble::tibble-package]. See help for `sp_get_table()` for a key to the columns.
+#' @family Detailed workflow
+#' @examples
+#' \dontrun{
+#' ds <- sp_get_dataset("rozv", 2018, 12)
+#' tf <- sp_get_table_file("balance-sheet", ds)
+#' sp_load_table(tf)
+#' }
+#' @export
+sp_load_table <- function(path, ico = NULL) {
+
+  ui_info("Reading data...")
+  suppressWarnings(suppressMessages(
+    dt <- readr::read_csv2(path, col_types = readr::cols(.default = readr::col_character()))))
+  # print(head(dt))
+  ui_info("Transforming data...")
+  if(max(stringr::str_length(dt$`ZC_ICO:ZC_ICO`), na.rm = TRUE) == 10) {
+    dt <- dplyr::mutate(dt, `ZC_ICO:ZC_ICO` = stringr::str_sub(.data$`ZC_ICO:ZC_ICO`, 3, 10))
+  }
+  if(!is.null(ico)) dt <- dt[dt$`ZC_ICO:ZC_ICO` %in% ico,]
+  dt <- dt %>%
+    purrr::set_names(stringr::str_remove(names(dt), "^[A-Z_0-9/]*:")) %>%
+    dplyr::mutate_at(dplyr::vars(dplyr::starts_with("ZU_")), ~switch_minus(.) %>% as.numeric(.)) %>%
+    tidyr::extract(.data$`0FISCPER`, c("vykaz_year", "vykaz_month"), "([0-9]{4})0([0-9]{2})") %>%
+    dplyr::mutate(vykaz_date = lubridate::make_date(.data$vykaz_year, .data$vykaz_month),
+                  vykaz_date = lubridate::make_date(.data$vykaz_year, .data$vykaz_month,
+                                                      lubridate::days_in_month(.data$vykaz_date))) %>%
+    dplyr::mutate_at(dplyr::vars(dplyr::ends_with("_date")), lubridate::dmy) %>%
+    dplyr::rename_all(dplyr::recode,
+                      ZCMMT_ITM = "polozka",
+                      ZC_VYKAZ = "vykaz",
+                      ZC_POLVYK = "polvyk",
+                      ZC_VTAB = "vtab",
+                      ZC_UCJED = "ucjed",
+                      ZFUNDS_CT = "finmisto",
+                      ZC_FUND = 'zdroj',
+                      `0FM_AREA` = "kapitola",
+                      `0CI_TYPE` = "polozka_typ",
+                      `0PU_MEASURE` = "rozprog",
+                      ZC_ICO = "ico",
+                      ZC_KRAJ = "kraj",
+                      ZC_NUTS = "nuts",
+                      ZC_LAU = "okres",
+                      ZC_PSUK = "psuk",
+                      ZC_UCRIS = "ucris",
+                      ZC_EDS = "eds",
+                      ZC_PVS = "pvs",
+                      ZC_ZREUZ = "ucelznak",
+                      ZU_MONET = "previous_net",
+                      ZU_AOBTTO = "current_gross",
+                      ZU_AONET = "current_net",
+                      ZU_AOKORR = "current_correction",
+                      ZU_HLCIN = "previous_core",
+                      ZU_HOSCIN = "previous_economic",
+                      ZU_HLCIBO = "current_core",
+                      ZU_HCINBO = "current_economic",
+                      FUNC0AREA = "paragraf",
+                      ZU_ROZSCH = "budget_adopted",
+                      ZU_ROZPZM = "budget_amended",
+                      ZU_KROZP = "budget_final",
+                      ZU_ROZKZ = "budget_spending",
+                      ZU_OBLIG = "budget_oblig",
+                      ZU_STAVP = "before",
+                      ZU_STAVPO = "after",
+                      ZU_ZVYS = "increase",
+                      ZU_SNIZ = "decrease",
+                      ZU_BEZUO = "current",
+                      ZU_SYNUC = "synuc",
+                      `0FUNC_AREA` = "paragraf")
+  return(dt)
+}
+
 #' Get a statnipokladna table
 #'
 #' Cleans and loads a table. If needed, a dataset containing the table is downloaded.
@@ -53,13 +177,13 @@ sp_tables_i <- tibble::tribble(~table_num, ~report_num, ~id,   ~table_code,   ~d
 #'
 #' - all columns are given names that are human-readable and facilitato add codelists
 #' - ICO (org. IDs) are normalised as in some datasets they are padded with leading zeros
-#' - a period, per_yr and per_m columns are created to identify the time period
+#' - a vykaz_date, vykaz_year and vykaz_month columns are created to identify the time period
 #' - value columns are transformed into numeric
 #' - other columns are left as character to avoid losing information
 #'
 #' ## Correspondence between input and output columns
 #'
-#' **Shared**
+#' **Shared/multiple tables**
 #'
 #'| Original | Output | English | Czech | Note |
 #'  | --- | --- | --- | --- | --- |
@@ -71,6 +195,7 @@ sp_tables_i <- tibble::tribble(~table_num, ~report_num, ~id,   ~table_code,   ~d
 #'  | ZC_FUND | zdroj | funding source | zdroj |  - |
 #'  | ZC_KRAJ | kraj | region | kraj |  - |
 #'  | ZC_NUTS | nuts | NUTS code | NUTS kód |  - |
+#'  | ZC_LAU | okres | NUTS code of LAU1 unit | LAU1 kód | - |
 #'
 #'  **Tables `budget-*`**
 #'
@@ -78,13 +203,33 @@ sp_tables_i <- tibble::tribble(~table_num, ~report_num, ~id,   ~table_code,   ~d
 #'  | --- | --- | --- | --- | --- |
 #'  | ZCMMT_ITM | polozka |  item/line |  položka (druhové členění) | NB: polozka != polvyk |
 #'  | 0FM_AREA | kapitola | chapter  |  kapitola | - |
+#'  | 0CI_TYPE | polozka_type | item/line type  |  typ položky | - |
 #'  | FUNC0AREA | paragraf |  sector line | paragraf (odvětvové členění)  |  - |
+#'  | ZC_PVS | pvs | programme code | programové výdaje státu | post-2014 |
+#'  | ZC_EDS | eds | subsidy and proprty evidence | Evidenční dotační systém / správa majetku ve vlastnictví státu | post-2014, No codelist - perhaps external via http://www.edssmvs.cz/DocumentsList.aspx?Agenda=CEIS |
+#'  | ZC_UCRIS | ucris | purpose | Účel | post-2014, no codelist found |
 #'  | 0FUNC_AREA | paragraf |  sector line | paragraf (odvětvové členění)  |  - |
 #'  | ZU_ROZSCH | budget_adopted |  budget as originally adopted | schválený rozpočet  |  - |
 #'  | ZU_ROZPZM | budget_amended |  budget as amended throughout the year | rozpočet po změnách  |  - |
 #'  | ZU_KROZP | budget_final |  final budget |  konečný rozpočet | - |
+#'  | ZU_OBLIG | budget_oblig | ? | obligo | - |
 #'  | ZU_ROZKZ | budget_spending |  actual spending | skutečnost  |  - |
 #'
+#'  **Table `budget-indicators`**
+#'
+#'  | Original | Output | English | Czech | Note |
+#'  | ZC_PSUK | psuk | budgetary indicator | Závazný a průřezový indikátor | Use `psuk` codelist |
+#'
+#' **Table `budget-central-old-subsidies` **
+#'
+#'  | Original | Output | English | Czech | Note |
+#'  | ZC_ZREUZ | ucelznak | Purpose identifier | Účelový znak | - |
+#'
+#' **Table `budget-central-old-purpose-grants` **
+#'
+#'  | Original | Output | English | Czech | Note |
+#'  | 0PU_MEASURE | rozprog | Budgetary programme | Rozpočtový program | - |
+#'
 #'  **Tables `balance-sheet*`**
 #'
 #'  | Original | Output | English | Czech | Note |
@@ -121,8 +266,8 @@ sp_tables_i <- tibble::tribble(~table_num, ~report_num, ~id,   ~table_code,   ~d
 #'  | ZU_BEZUO | current | current period |  běžné účetní období |  - |
 #'
 #'
-#' @param table_id A table ID. See `id` column in `sp_tables` for a list of available codelists.
-#' @param year year, numeric, 2015-2018 for some datasets, 2010-2018 for others.
+#' @param table_id A table ID. See `id` column in `sp_tables` for a list of available tables
+#' @param year year, numeric, 2015-2019 for some datasets, 2010-2020 for others.
 #' Can be a vector of length > 1 (see Details for how to work with data across time periods.).
 #' @param month month, numeric. Must be 3, 6, 9 or 12. Can be a vector of length > 1 (see details).
 #' @param ico ID(s) of org to return, character of length one or more. If unset, returns all orgs. ID not checked for correctness/existence. See <http://monitor.statnipokladna.cz/datovy-katalog/prohlizec-ciselniku/ucjed> to look up ID of any org in the dataset.
@@ -133,8 +278,7 @@ sp_tables_i <- tibble::tribble(~table_num, ~report_num, ~id,   ~table_code,   ~d
 #' @return a [tibble][tibble::tibble-package]; see Details for key to the columns
 #' @encoding UTF-8
 #' @examples
-#' \donttest{
-#' allorgs_latest <- sp_get_table("budget-central")
+#' \dontrun{
 #' allorgs_2018 <- sp_get_table("budget-central", 2018)
 #' allorgs_mid2018 <- sp_get_table("budget-central", 2018, 6)
 #' oneorg_multiyear <- sp_get_table("budget-central", 2017:2018, 12, ico = "00064581")
@@ -143,91 +287,44 @@ sp_tables_i <- tibble::tribble(~table_num, ~report_num, ~id,   ~table_code,   ~d
 #' @export
 #' @family Core workflow
 #'
-sp_get_table <- function(table_id, year = 2018, month = 12, ico = NULL,
-                      redownload = FALSE, dest_dir = NULL) {
+sp_get_table <- function(table_id, year, month = 12, ico = NULL,
+                         redownload = FALSE, dest_dir = NULL) {
   stopifnot(is.character(ico) | is.null(ico))
   if(interactive() == FALSE & (missing(year) | missing(month))) {
-    usethis::ui_warn("Either {usethis::ui_field('year')} or {usethis::ui_field('month')} not set.
-                     Using defaults of {usethis::ui_value(year)} and {usethis::ui_value(month)}.")
-    usethis::ui_todo("Set these values explicitly for reproducibility as the defaults may change in the future
-                     to provide access to the latest data by default.")
+    if(missing(year)) {
+      ui_stop("{ui_field('year')} not set. Please set a value.")
 
+    } else if(missing(month)) {
+      ui_warn("{ui_field('month')} not set. Using default of {ui_value(month)}.")
+
+    }
+
+    ui_todo("Set period parameters explicitly for reproducibility.")
   }
 
   if(is.null(dest_dir)) dest_dir <- getOption("statnipokladna.dest_dir",
                                               default = tempdir())
 
-  if(!(table_id %in% sp_tables_i$id)) usethis::ui_stop("Not a valid table id. Consult {usethis::ui_code('sp_tables')}.")
+  if(!(table_id %in% sp_tables_i$id)) ui_stop("Not a valid table id. Consult {ui_code('sp_tables')}.")
   dataset_id <- sp_tables_i$dataset_id[sp_tables_i$id == table_id]
-  table_regex <- paste0(sp_tables_i$file_regex[sp_tables_i$id == table_id])
-  get_one_table <- function(dataset_id, year = year, month = month, dest_dir = dest_dir,
-                            redownload = redownload) {
-    dslist <- sp_get_dataset(dataset_id, year = year, month = month, dest_dir = dest_dir,
-                          redownload = redownload)
-    table_file <- dslist[stringr::str_detect(dslist,
-                                             paste0(table_regex, "(_[0-9]*)?\\.(csv|CSV)"))]
-    stopifnot(length(table_file) == 1)
-    usethis::ui_info("Reading data...")
-    suppressWarnings(suppressMessages(
-      dt <- readr::read_csv2(table_file, col_types = readr::cols(.default = readr::col_character()))))
-    # print(head(dt))
-    usethis::ui_info("Transforming data...")
-    if(max(stringr::str_length(dt$`ZC_ICO:ZC_ICO`), na.rm = TRUE) == 10) {
-      dt <- dplyr::mutate(dt, `ZC_ICO:ZC_ICO` = stringr::str_sub(.data$`ZC_ICO:ZC_ICO`, 3, 10))
-    }
-    if(!is.null(ico)) dt <- dt[dt$`ZC_ICO:ZC_ICO` %in% ico,]
-    dt <- dt %>%
-      purrr::set_names(stringr::str_remove(names(dt), "^[A-Z_0-9/]*:")) %>%
-      dplyr::mutate_at(dplyr::vars(dplyr::starts_with("ZU_")), ~switch_minus(.) %>% as.numeric(.)) %>%
-      tidyr::extract(.data$`0FISCPER`, c("per_yr", "per_m"), "([0-9]{4})0([0-9]{2})") %>%
-      dplyr::mutate(period_vykaz = lubridate::make_date(.data$per_yr, .data$per_m),
-                    period_vykaz = lubridate::make_date(.data$per_yr, .data$per_m,
-                                                        lubridate::days_in_month(.data$period_vykaz))) %>%
-      dplyr::mutate_at(dplyr::vars(dplyr::ends_with("_date")), lubridate::dmy) %>%
-      dplyr::rename_all(dplyr::recode,
-                        ZCMMT_ITM = "polozka",
-                        ZC_VYKAZ = "vykaz",
-                        ZC_POLVYK = "polvyk",
-                        ZC_VTAB = "vtab",
-                        ZC_UCJED = "ucjed",
-                        ZFUNDS_CT = "finmisto",
-                        ZC_FUND = 'zdroj',
-                        `0FM_AREA` = "kapitola",
-                        ZC_ICO = "ico",
-                        ZC_KRAJ = "kraj",
-                        ZC_NUTS = "nuts",
-                        ZU_MONET = "previous_net",
-                        ZU_AOBTTO = "current_gross",
-                        ZU_AONET = "current_net",
-                        ZU_AOKORR = "current_correction",
-                        ZU_HLCIN = "previous_core",
-                        ZU_HOSCIN = "previous_economic",
-                        ZU_HLCIBO = "current_core",
-                        ZU_HCINBO = "current_economic",
-                        FUNC0AREA = "paragraf",
-                        ZU_ROZSCH = "budget_adopted",
-                        ZU_ROZPZM = "budget_amended",
-                        ZU_KROZP = "budget_final",
-                        ZU_ROZKZ = "budget_spending",
-                        ZU_STAVP = "before",
-                        ZU_STAVPO = "after",
-                        ZU_ZVYS = "increase",
-                        ZU_SNIZ = "decrease",
-                        ZU_BEZUO = "current",
-                        ZU_SYNUC = "synuc",
-                        `0FUNC_AREA` = "paragraf")
-    return(dt)
-  }
-  years_months <- expand.grid(y = year, m = month)
-  dt_fin <- purrr::map2_dfr(years_months$y, years_months$m,
-                            ~get_one_table(dataset_id, .x, .y, dest_dir = dest_dir,
-                                           redownload = redownload))
-  # onyr <- c(2018) %>% purrr::map_dfr(~ sp_get_table(51101, year = ., month = 12))
 
+  downloaded_datasets <- sp_get_dataset(dataset_id, year, month, dest_dir, redownload)
+
+  return_table <- function(dataset_path, table_id, ico) {
+    table_file_path <- sp_get_table_file(table_id, dataset_path, reunzip = redownload)
+    tbl <- sp_load_table(table_file_path, ico)
+    return(tbl)
   }
+
+  dt_fin <- purrr::map_dfr(downloaded_datasets, return_table, table_id, ico)
+  # onyr <- c(2018) %>% purrr::map_dfr(~ sp_get_table(51101, year = ., month = 12))
+  return(dt_fin)
+}
 # onyr <- c(2018) %>% purrr::map_dfr(~ sp_get_table(2, year = ., month = 12))
 # onyr <- c(2018) %>% purrr::map_dfr(~ sp_get_table(1, year = ., month = 12))
 
+# Deprecated --------------------------------------------------------------
+
 #' Deprecated: Get a statnipokladna table\cr\cr
 #' Deprecated, use `sp_get_table()` instead.
 #'
@@ -237,7 +334,7 @@ sp_get_table <- function(table_id, year = 2018, month = 12, ico = NULL,
 #'
 #' @return a [tibble][tibble::tibble-package]
 #' @export
-get_table <- function(table_id, year = 2018, month = 12, ico = NULL,
+get_table <- function(table_id, year, month = 12, ico = NULL,
                       redownload = FALSE, dest_dir = NULL) {
   lifecycle::deprecate_warn("0.5.2", "statnipokladna::get_table()", "sp_get_table()")
 }
